<?php
namespace EMS\PortalFuncionario\Geral\Services;

use Zeedhi\Framework\Controller\Crud;
use Doctrine\ORM\EntityManager;
use Ems\PortalFuncionario\Util\HcmServices;

use EMS\PortalFuncionario\Util\DateUtil;

use EMS\PortalFuncionario\Geral\Repository\{OcorrenciaRepository, MarcacaoRepository};

class OcorrenciaService extends Crud {

    protected const ocorrenciaDefaults = array(
        'ALMOCO' => array(
            'nrtipoprogocorrencia' => '20'
        )
    );

    protected const messages = array(
        'ERRO_OCORRENCIA_ALMOCO_INSERIDA' => 'Ocorrência de almoço já cadastratada para o dia @.',
    );

    public function __construct(EntityManager $entityManager, MarcacaoRepository $marcacaoRepository, OcorrenciaRepository $ocorrenciaRepository, HcmServices $hcmServices) {
        $this->entityManager        = $entityManager;
        $this->marcacaoRepository   = $marcacaoRepository;
        $this->ocorrenciaRepository  = $ocorrenciaRepository;
        $this->hcmServices = $hcmServices;
    }

    /**
     * Função responsável pela criação de uma ocorrência relacionada a ausência de almoço durante a jornada
     */
    public function setOcorrenciaAlmoco(string $nrvinculom, string $nrorg, string $nrmarcacaoponto, string $cdoperador, string $dia = null) : int {
        try {
            if (empty($dia)) {
                $dataAtual = new \DateTime();
                $dia = $dataAtual->format('d/m/Y');
            }
            
            $nrtipoprogocorrencia = self::ocorrenciaDefaults['ALMOCO']['nrtipoprogocorrencia'];
            $dtinicioprogocor = $dia;
            
            $existemOcorrenciasAlmocoDia = $this->existemOcorrenciasAlmoco($nrvinculom, $nrorg, $dtinicioprogocor);
            
            if ($existemOcorrenciasAlmocoDia) {
                throw new \Exception(str_replace('@', $dtinicioprogocor, self::messages['ERRO_OCORRENCIA_ALMOCO_INSERIDA']));
            }
            
            $idprogocorconfirmada = $this->ocorrenciaRepository->getTipoProgocorrencia($nrorg, $nrtipoprogocorrencia);
            $dadosHorarioVinculo = $this->marcacaoRepository->getDadosHorarioVinculo($nrvinculom, $nrorg);
            $dtfimprogocor = $dia;
            $iddiassemana = DateUtil::getIdWeekDay();
            $idtipohorario = $dadosHorarioVinculo['IDTIPOHORARIO'];
            $nrhordiam = $dadosHorarioVinculo['NRHORDIAM'];
            $nrintervalohorario = $dadosHorarioVinculo['NRINTERVALOHORARIO'];
            $idtipocalendario = '';
            $nrmotivoocorfreq = '';
            $qtmaximaprogocor = '';
            $dsobservacao = '';
            $dsobservacaorescusa = '';
            $hrprojetarintervalonprog = '';
            $dtinclusao = $dia;

            return $this->ocorrenciaRepository->geraOcorrenciaFrequencia(
                $nrvinculom,
                $dtinicioprogocor,
                $dtfimprogocor,
                $iddiassemana,
                $idtipocalendario,
                $idtipohorario,
                $nrhordiam,
                $nrintervalohorario,
                $nrmotivoocorfreq,
                $qtmaximaprogocor,
                $dsobservacao,
                $nrtipoprogocorrencia,
                $nrmarcacaoponto,
                $idprogocorconfirmada,
                $dsobservacaorescusa,
                $hrprojetarintervalonprog,
                $dtinclusao,
                $cdoperador,
                $nrorg
            );
            
        } catch (\Exception $e) {
            throw $e;
        }
    }

    // NAO UTILIZADO; Ideia pra unificar o registro das ocorrências
    public function buildNovaOcorrencia(string $nrvinculom, string $nrorg, string $nrmarcacaoponto, string $nrtipoprogocorrencia, string $cdoperador) {
        $data = new \DateTime();
        $dataAtual = $data->format('d/m/Y');
        $dadosHorarioVinculo = $this->marcacaoRepository->getDadosHorarioVinculo($nrvinculom, $nrorg);
        $novaOcorrencia = array(
            'nrvinculom'                => $nrvinculom,
            'nrtipoprogocorrencia'      => self::ocorrenciaDefaults['ALMOCO']['nrtipoprogocorrencia'],
            'idprogocorconfirmada'      => $this->ocorrenciaRepository->getTipoProgocorrencia($nrorg, $nrtipoprogocorrencia),
            'dtinicioprogocor'          => $dataAtual,
            'dtfimprogocor'             => $dataAtual,
            'iddiassemana'              => DateUtil::getIdWeekDay(),
            'idtipohorario'             => $dadosHorarioVinculo['IDTIPOHORARIO'],
            'nrhordiam'                 => $dadosHorarioVinculo['NRHORDIAM'],
            'nrintervalohorario'        => $dadosHorarioVinculo['NRINTERVALOHORARIO'],
            'idtipocalendario'          => '',
            'nrmotivoocorfreq'          => '',
            'qtmaximaprogocor'          => '',
            'dsobservacao'              => '',
            'dsobservacaorescusa'       => '',
            'hrprojetarintervalonprog'  => '',
            'dtinclusao'                => $dataAtual,
            'nrmarcacaoponto'           => $nrmarcacaoponto,
            'cdoperador'                => $cdoperador,
        );

        return $novaOcorrencia;
    }

    public function getOcorrenciasPendentes(string $nrorg, string $nrvinculom, string $nrperiodoapuracao) {
        try {
			return $this->ocorrenciaRepository->getOcorrencias(
                $nrorg,
                $nrvinculom,
                'P',
                $nrperiodoapuracao
            );
            
		} catch (\Exception $e) {
			throw $e;
		}        
    }

    public function getOcorrenciasPeriodo(string $nrorg, string $nrvinculom, string $nrperiodoapuracao) {
        try {
            $params = [
                'requestType' => 'Row',
                'row' => [
                    'NRVINCULOM'      => $nrvinculom,
                    'NRPERIODOAPURACAO' => $nrperiodoapuracao,
                    'NRORG'     => $nrorg
                ]
            ];

            $result = $this->hcmServices->send('getOcorrenciasPeriodo', $params, $nrorg, "POST");

            return $result['dataset']['getOcorrenciasPeriodo'];

            /*
            return $this->ocorrenciaRepository->getOcorrenciasPeriodo(
                $nrorg,
                $nrvinculom,
                $nrperiodoapuracao
            );*/
		} catch (\Exception $e) {
			throw $e;
		}        
    }

    public function getMotivoOcorFreq(string $nrorg) {
        try {
			return $this->ocorrenciaRepository->getMotivoOcorFreq(
                $nrorg
            );
		} catch (\Exception $e) {
			throw $e;
		}        
    }

    public function setOcorrenciaFreq($row) {
        try {
            $params = [
                'requestType' => 'Row',
                'row' => $row
            ];
            $result = $this->hcmServices->send('setOcorrenciaFreq', $params, $row['NRORG']);
            return $result;
            /*if($row['__is_new']) {
                return $this->ocorrenciaRepository->newOcorrenciaFreq($row);
            }else {
                return $this->ocorrenciaRepository->updateOcorrenciaFreq($row);
            }*/
		} catch (\Exception $e) {
			throw $e;
		}        
    }

    public function setOcorrenciasFreq(
        $row
    ): bool {

        $hcmServicesRoute = 'setOcorrenciasFreq';

        $nrOrg = $row['NRORG'];

        $params = [
            'requestType' => 'Row',
            'row' => $row
        ];

        $response = $this->hcmServices->send($hcmServicesRoute, $params, $nrOrg);
        $success = $response['dataset'][$hcmServicesRoute]['success'];

        return $success;
    }

    public function removeOcorrenciaFreq($row) {
        try {
            $params = [
                'requestType' => 'Row',
                'row' => $row
            ];
            $result = $this->hcmServices->send('removeOcorrenciaFreq', $params, $row['NRORG']);
            //return $this->ocorrenciaRepository->removeOcorrenciaFreq($row);
            return $result;
		} catch (\Exception $e) {
			throw $e;
		}        
    }

    public function removeOcorrenciasFreq(
        $row
    ): bool {

        $hcmServicesRoute = 'removeOcorrenciasFreq';

        $nrOrg = $row['NRORG'];

        $params = [
            'requestType' => 'Row',

            // ver se precisa passar a row toda
            'row' => $row
        ];

        $response = $this->hcmServices->send($hcmServicesRoute, $params, $nrOrg);
        $success = $response['dataset'][$hcmServicesRoute]['success'];

        return $success;
    }

    public function removeAlmocos(
        $nrorg,
        $nrvinculom,
        $date
    ): bool {

        $hcmServicesRoute = 'removeAlmocos';

        $params = [
            'requestType' => 'Row',
            'row' => [
                'NRVINCULOM'    => $nrvinculom,
                'DATE'          => $date
            ]
        ];

        $response = $this->hcmServices->send($hcmServicesRoute, $params, $nrorg);
        $success = $response['dataset'][$hcmServicesRoute]['success'];

        return $success;
    }

    public function existemOcorrenciasAlmoco(string $nrvinculom, string $nrorg, string $dtinicioprogocor): bool {
        $nrtipoprogocorrencia = self::ocorrenciaDefaults['ALMOCO']['nrtipoprogocorrencia'];
        $ocorrenciasAlmocoDia = $this->ocorrenciaRepository->getOcorrenciasPorTipoOcorrencia($nrvinculom, $nrorg, $nrtipoprogocorrencia, $dtinicioprogocor);
        
        $existemOcorrenciasAlmocoDia = !empty($ocorrenciasAlmocoDia);
        
        return $existemOcorrenciasAlmocoDia;
    }

    public function getTpOcorrencia($nrOrg) {

        $params = [
            'requestType' => 'Row',
            'row' => [
                'NRORG'   => $nrOrg
            ]
        ];

        $result = $this->hcmServices->send('getTpOcorrencia', $params, $nrOrg);
        $paresMarcacoes = $result['dataset']['getTpOcorrencia'];

        return $paresMarcacoes;
    }

    public function insertAlmocoMarcacaoRetroativa(
        int     $nrOrg,
        string  $cdOperador,
        int     $nrVinculo,
        array   $dates
    ): bool {

        $hcmServicesRoute = 'insertAlmocoMultiplesGestor';

        $params = [
            'requestType' => 'Row',
            'row' => [
                'NRVINCULOM'        => $nrVinculo,
                'CDOPERADOR'        => $cdOperador,
                'DATE'              => $dates
            ]
        ];

        $response = $this->hcmServices->send($hcmServicesRoute, $params, $nrOrg);
        $success = $response['dataset'][$hcmServicesRoute]['success'];

        return $success;
    }

    public function getMotivosOcorrencia($row): array
    {
        $params = [
            'P_NRORG' => $row['NRORG']
        ];

        $motivos = $this->ocorrenciaRepository->getMotivosOcorrencia($params);

        return $motivos;
    }

    public function insereOcorrenciasAlmoco(
        int     $nrOrg,
        int     $nrVinculo,
        string  $cdOperador,
        string  $horaMarcacão,
        array   $dia
    ) : array {

        $params = [
            'requestType' => 'Row',
            'row' => [
                'NRVINCULOM'        => $nrVinculo,
                'HORAMARCACAO'      => $horaMarcacão,
                'CDOPERADOR'        => $cdOperador,
                'DATE'              => $dia
            ]
        ];

        $result = $this->hcmServices->send('insereOcorrenciasAlmoco', $params, $nrOrg);
        $insereOcorrenciasAlmoco = $result['dataset']['insereOcorrenciasAlmoco'];

        return $insereOcorrenciasAlmoco;
    }

    public function solicitaCompensacaoFaltas(
        int     $nrOrg,
        int     $nrVinculo,
        array   $dia
    ): array {
        $route = 'solicitaCompensacaoFaltas';

        $params = [
            'requestType' => 'Row',
            'row' => [
                'NRVINCULOM'        => $nrVinculo,
                'DATE'              => $dia
            ]
        ];

        $result = $this->hcmServices->send($route, $params, $nrOrg);
        $status = $result['dataset'][$route];

        return $status;
    }

    public function solicitaAbonoFaltas(
        int     $nrOrg,
        int     $nrVinculo,
        array   $dia,
        string  $dsobservacao,
        string  $file
    ) : array {

        $params = [
            'requestType' => 'Row',
            'row' => [
                'NRVINCULOM'        => $nrVinculo,
                'DATE'              => $dia,
                'DSOBSERVACAO'      => $dsobservacao,
                'FILE'              => $file,
            ]
        ];

        $result = $this->hcmServices->send('solicitaAbonoFaltas', $params, $nrOrg);
        $solicitaAbonoFaltas = $result['dataset']['solicitaAbonoFaltas'];

        return $solicitaAbonoFaltas;
    }

    public function solicitarCompensacaoHoras(
        int $nrOrg,
        int $nrVinculo,
        string $date
    ): array {

        $hcmServicesRoute = 'solicitarCompensacaoHoras';

        $params = [
            'requestType' => 'Row',
            'row' => [
                'NRVINCULOM'        => $nrVinculo,
                'DATE'              => $date
            ]
        ];

        $response = $this->hcmServices->send($hcmServicesRoute, $params, $nrOrg);
        $status = $response['dataset'][$hcmServicesRoute];

        return $status;
    }

    public function solicitarHorasExtras(
        int    $nrOrg,
        int    $nrVinculo,
        string $observacao,
        string $date
    ): array {
        $hcmServicesRoute = 'solicitarHorasExtras';

        $params = [
            'requestType' => 'Row',
            'row' => [
                'NRVINCULOM'        => $nrVinculo,
                'DSOBSERVACAO'      => $observacao,
                'DATE'              => $date
            ]
        ];

        $response = $this->hcmServices->send($hcmServicesRoute, $params, $nrOrg);
        $status = $response['dataset'][$hcmServicesRoute];

        return $status;
    }

    public function getOcorrenciaFile(
        int $nrOrg,
        int $nrOcorrencia
    ): array {
        $route = 'getOcorrenciaFile';

        $params = [
            'requestType' => 'Row',
            'row' => [
                'NRORG'            => $nrOrg,
                'NRPROGOCORRENCIA' => $nrOcorrencia
            ]
        ];

        $result = $this->hcmServices->send($route, $params, $nrOrg);
        $file = $result['dataset'][$route];

        return $file;
    }
}
